@@ -28,14 +28,12 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <unistd.h>
 
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-/* for fast user switching */
-#include <libgnomevfs/gnome-vfs-init.h>
-
 #include "gs-lock-plug.h"
 
 #include "passwd.h"
@@ -43,6 +41,9 @@
 
 
 /* Profiling stuff adapted from gtkfilechooserdefault */
+/* To use run:
+ *  strace -ttt -f -o logfile.txt ./gnome-screensaver-dialog
+ */
 
 #undef PROFILE_LOCK_DIALOG
 #ifdef PROFILE_LOCK_DIALOG
@@ -64,20 +65,18 @@ _gs_lock_plug_profile_log (const char *func,
                            const char *msg1,
                            const char *msg2)
 {
-        char    *str;
-        GTimeVal now;
+        char *str;
 
         if (indent < 0)
                 profile_add_indent (indent);
 
-        g_get_current_time (&now);
-
         if (profile_indent == 0)
-                str = g_strdup_printf ("MARK %ld.%6.6ld: %s: %s %s %s", now.tv_sec, now.tv_usec, G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+                str = g_strdup_printf ("MARK: %s: %s %s %s", G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
         else
-                str = g_strdup_printf ("MARK %ld.%6.6ld: %s: %*c %s %s %s", now.tv_sec, now.tv_usec, G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+                str = g_strdup_printf ("MARK: %s: %*c %s %s %s", G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+
+        access (str, F_OK);
 
-        fprintf (stderr, "%s\n", str);
         g_free (str);
 
         if (indent > 0)
@@ -94,10 +93,11 @@ _gs_lock_plug_profile_log (const char *func,
 #endif
 
 
-static gboolean verbose       = FALSE;
-static gboolean show_version  = FALSE;
-static gboolean enable_logout = FALSE;
-static gboolean enable_switch = FALSE;
+static gboolean verbose        = FALSE;
+static gboolean show_version   = FALSE;
+static gboolean enable_logout  = FALSE;
+static gboolean enable_switch  = FALSE;
+static char    *logout_command = NULL;
 
 static GOptionEntry entries [] = {
         { "verbose", 0, 0, G_OPTION_ARG_NONE, &verbose,
@@ -106,6 +106,10 @@ static GOptionEntry entries [] = {
           N_("Version of this application"), NULL },
         { "enable-logout", 0, 0, G_OPTION_ARG_NONE, &enable_logout,
           N_("Show the logout button"), NULL },
+        { "logout-command", 0, 0, G_OPTION_ARG_STRING, &logout_command,
+          N_("Command to invoke from the logout button"), NULL },
+        { "enable-switch", 0, 0, G_OPTION_ARG_NONE, &enable_switch,
+          N_("Show the switch user button"), NULL },
         { NULL }
 };
 
@@ -186,6 +190,10 @@ popup_dialog_idle (void)
                 g_object_set (widget, "logout-enabled", TRUE, NULL);
         }
 
+        if (logout_command) {
+                g_object_set (widget, "logout-command", logout_command, NULL);
+        }
+
         if (enable_switch) {
                 g_object_set (widget, "switch-enabled", TRUE, NULL);
         }
@@ -207,7 +215,7 @@ popup_dialog_idle (void)
  * Copyright (c) 2005 William Jon McCann <mccann@jhu.edu>
  *
  * Initializations that potentially take place as a priveleged user:
-   If the xscreensaver executable is setuid root, then these initializations
+   If the executable is setuid root, then these initializations
    are run as root, before discarding privileges.
 */
 static gboolean
@@ -351,10 +359,6 @@ main (int    argc,
                 exit (1);
         }
 
-#ifdef HAVE_USER_SWITCHING
-        enable_switch = TRUE;
-#endif
-
         if (show_version) {
                 g_print ("%s %s\n", argv [0], VERSION);
                 exit (1);
@@ -369,10 +373,6 @@ main (int    argc,
                 exit (1);
         }
 
-        /* for fast user switching */
-        g_thread_init (NULL);
-        gnome_vfs_init ();
-
         g_idle_add ((GSourceFunc)popup_dialog_idle, NULL);
 
         gtk_main ();
