@@ -82,7 +82,7 @@ struct GSWindowPrivate
 };
 
 enum {
-        DISACTIVATED,
+        DEACTIVATED,
         DIALOG_UP,
         DIALOG_DOWN,
         LAST_SIGNAL
@@ -100,7 +100,7 @@ enum {
 static GObjectClass   *parent_class = NULL;
 static guint           signals [LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (GSWindow, gs_window, GTK_TYPE_WINDOW);
+G_DEFINE_TYPE (GSWindow, gs_window, GTK_TYPE_WINDOW)
 
 static void
 set_invisible_cursor (GdkWindow *window,
@@ -308,9 +308,9 @@ gs_window_get_gdk_window (GSWindow *window)
 }
 
 static gboolean
-emit_disactivated_idle (GSWindow *window)
+emit_deactivated_idle (GSWindow *window)
 {
-        g_signal_emit (window, signals [DISACTIVATED], 0);
+        g_signal_emit (window, signals [DEACTIVATED], 0);
 
         return FALSE;
 }
@@ -327,6 +327,7 @@ spawn_on_window (GSWindow *window,
         char      **argv;
         char       *str;
         GPtrArray  *env;
+        GError     *error;
         gboolean    result;
         GIOChannel *channel;
         int         standard_output;
@@ -334,8 +335,12 @@ spawn_on_window (GSWindow *window,
         int         id;
         int         i;
 
-        if (!g_shell_parse_argv (command, &argc, &argv, NULL))
+        error = NULL;
+        if (! g_shell_parse_argv (command, &argc, &argv, &error)) {
+                g_warning ("Could not parse command: %s", error->message);
+                g_error_free (error);
                 return FALSE;
+        }
 
         env = g_ptr_array_new ();
 
@@ -365,6 +370,7 @@ spawn_on_window (GSWindow *window,
                                                        g_getenv ("LANGUAGE")));
         g_ptr_array_add (env, NULL);
 
+        error = NULL;
         result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
                                                  g_get_home_dir (),
                                                  argv,
@@ -376,13 +382,15 @@ spawn_on_window (GSWindow *window,
                                                  NULL,
                                                  &standard_output,
                                                  NULL,
-                                                 NULL);
+                                                 &error);
 
         for (i = 0; i < env->len; i++)
                 g_free (g_ptr_array_index (env, i));
         g_ptr_array_free (env, TRUE);
 
         if (! result) {
+                g_warning ("Could not start command '%s': %s", command, error->message);
+                g_error_free (error);
                 g_strfreev (argv);
                 return FALSE;
         }
@@ -629,7 +637,7 @@ command_watch (GIOChannel   *source,
                 gs_window_dialog_finish (window);
 
                 if (window->priv->dialog_response == DIALOG_RESPONSE_OK) {
-                        g_idle_add ((GSourceFunc)emit_disactivated_idle, window);
+                        g_idle_add ((GSourceFunc)emit_deactivated_idle, window);
                 }
 
                 gs_window_clear (window);
@@ -719,7 +727,7 @@ gs_window_request_unlock (GSWindow *window)
                 return;
 
         if (! window->priv->lock_enabled) {
-                g_idle_add ((GSourceFunc)emit_disactivated_idle, window);
+                g_idle_add ((GSourceFunc)emit_deactivated_idle, window);
                 return;
         }
 
@@ -1045,11 +1053,11 @@ gs_window_class_init (GSWindowClass *klass)
 
         g_type_class_add_private (klass, sizeof (GSWindowPrivate));
 
-        signals [DISACTIVATED] =
-                g_signal_new ("disactivated",
+        signals [DEACTIVATED] =
+                g_signal_new ("deactivated",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSWindowClass, disactivated),
+                              G_STRUCT_OFFSET (GSWindowClass, deactivated),
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__VOID,
